@@ -6,10 +6,10 @@ use base qw/DBIx::Class/;
 use Scalar::Util qw(blessed);
 use Class::C3::Componentised;
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 __PACKAGE__->mk_classdata('_i18n_columns');
-__PACKAGE__->mk_group_accessors( 'simple' => qw/ language _i18n_column_row / );
+__PACKAGE__->mk_group_accessors( 'simple' => qw/ _language _i18n_column_row / );
 
 =head1 NAME
 
@@ -17,7 +17,7 @@ DBIx::Class::I18NColumns - Internationalization for DBIx::Class Result class
 
 =head1 VERSION
 
-Version 0.10
+Version 0.11
 
 =cut
 
@@ -79,6 +79,8 @@ This module allows you to define columns that will store multiple values asociat
 
 This component will create a new resultset on your schema for each one that use it. The auto-created resultset will use the columns definition you give to add_i18n_columns() plus a FK and language columns. The i18n values of each language will reside in a row of this resultset and will transparently work as any other column as long as you provide a language to your row or resultset methods as you can see at the synopsis.
 
+Language will be propagated to relationships with result sources that also use this component.
+
 =head1 METHODS
 
 =head2 add_i18n_columns
@@ -118,6 +120,7 @@ sub add_i18n_columns {
     }
 
     $self->_create_i18n_result_source if $self->auto_i18n_rs;
+    $self->_add_schema_accessor;
     $self->_add_languages_column;
 }
 
@@ -125,6 +128,19 @@ sub add_i18n_columns {
 
 Get or set the language for the row.
 
+=cut
+sub language {
+    my $self = shift;
+
+    if ( my $value = shift ) {
+        $self->result_source->schema->_language_last_set($value);
+        return $self->_language($value);
+    }
+
+    return $self->_language if $self->_language;
+    return $self->_language($self->result_source->schema->_language_last_set);
+}
+
 =head2 i18n_resultset
 
 Retuns an instance of the resultset where i18n strings are stored.
@@ -165,6 +181,16 @@ sub languages {
     return wantarray ? @langs : [ @langs ];
 }
 
+=head2 has_language
+
+Check for a given language to be present on the row.
+
+=cut
+sub has_language {
+    my ( $self, $language ) = @_;
+    return grep { $_ eq $language } $self->languages;
+}
+
 =head2 language_column
     
 The name for the language column to be used and autocreated.
@@ -205,6 +231,7 @@ Returns true if the source has a i18n column of this name, false otherwise.
 
 sub has_i18n_column {
     my ( $self, $column ) = ( shift, shift );
+    return 0 unless $self->_i18n_columns;
     return ( exists $self->_i18n_columns->{$column} ) ? 1 : 0;
 }
 
@@ -295,7 +322,6 @@ sub get_column {
             });
         }
         return $self->_i18n_column_row->{$lang}->$column;
-        #TODO: make accessor work !
     }
 
     return $self->next::method( $column, @_ );
@@ -360,8 +386,23 @@ sub get_from_storage {
     return $new_object;
 }
 
-#TODO: delete
-#TODO: get_columns
+=head2 related_resultset
+
+Overloaded L<DBIx::Class::Relationship::Base/related_resultset> to propagate
+language to related resultsets that also use this component.
+
+=cut
+sub related_resultset {
+    my $self = shift;
+    my $lang = $self->language;
+
+    my $rel_rs = $self->next::method( @_ );
+
+    if ( $rel_rs && $rel_rs->can('language') && $lang ) {
+        $rel_rs->language($lang);
+    }
+    return $rel_rs;
+}
 
 sub _create_i18n_result_source {
     my $self = shift;
@@ -418,6 +459,24 @@ sub _add_languages_column {
     ) unless $self->has_column('languages');
 }
 
+# this is nasty but I can't find another method to keep
+# language after any of many_to_many() autogenerated methods,
+# this wont be used when it's possible.
+sub _add_schema_accessor {
+    my $self = shift;
+    my $schema_class = $self->schema_class;
+
+    no strict 'refs';
+    *{ $schema_class . '::_language_last_set' } = sub {
+        my $schema_obj = shift;
+        if ( my $set = shift ) {
+            $schema_obj->{_language_last_set} = $set;
+        }
+
+        return $schema_obj->{_language_last_set};
+    };
+}
+
 sub _make_languages_dirty {
     my ( $self, $lang ) = @_;
 
@@ -467,10 +526,6 @@ sub _i18n_class_moniker {
 
 =item *
 
-Overload delete, get_columns and get_dirty_columns
-
-=item *
-
 Make it possible to search by i18n_columns
 
 =item *
@@ -483,10 +538,6 @@ Make it possible to use this component on tables with multi-columns PK
 
 =back
 
-=head1 AUTHOR
-
-Diego Kuperman, C<< <diego at freekeylabs.com > >>
-
 =head1 BUGS
 
 This is an early release, so probable there are plenty of bugs around.
@@ -494,13 +545,16 @@ This is an early release, so probable there are plenty of bugs around.
 Please report any bugs or feature requests to C<bug-dbix-class-i18ncolumns at rt.cpan.org>, or through
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=DBIx-Class-I18NColumns>.  I will be notified, and then you'll automatically be notified of progress on your bug as I make changes.
 
+=head1 AUTHOR
+
+Diego Kuperman, C<< <diego at freekeylabs.com > >>
+
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
 
     perldoc DBIx::Class::I18NColumns
 
-
 You can also look for information at:
 
 =over 4
